/*
 BH1750irq.c
 * Software to accompany the article -
 * "The Quark D2000 Development Board: I2C Interfacing"
 * Interface a BH1750 light sensor module to the
 * D2000 board using irq-I2c
 *****************************************************
 ** This software is offered strictly as is with no **
 ** warranties whatsoever. Use it at your own risk. **
 *****************************************************
 */


#include "qm_common.h"
#include "qm_i2c.h"
#include "qm_pinmux.h"
#include "clk.h"
#include "qm_interrupt.h"
#include "qm_isr.h"


#define BYTES2READ (2)
#define BH1750addr (0x23) 	/* slave I2c address */

/* globals */
uint8_t BHlxdata[2]; 		/* data read from the sensor */
uint16_t lux; 				/* converted lux value       */
qm_i2c_status_t I2Cstatus; 	/* see i2c.h for values      */


/*  I2C structure variables  */
static qm_i2c_config_t I2Ccfg;
/* I2C asynch transfer structure */
static qm_i2c_transfer_t irq_write_xfer;
static qm_i2c_transfer_t irq_read_xfer;
/*I2C asynch structure vars */
static uint8_t id_write=0;
static uint8_t id_read=1;
/* callback function */
static void i2c_cb(void *data, int rc, qm_i2c_status_t status, uint32_t len);
/* call back will set this variable to true */
volatile bool i2cirq_complete=false;
/*__________________________________________________________________________*/
int wireBegin() {
	/* set up for I2C IRQ */
	qm_irq_request(QM_IRQ_I2C_0, qm_i2c_0_isr);
	/*  Enable I2C 0 */
	clk_periph_enable(CLK_PERIPH_CLK | CLK_PERIPH_I2C_M0_REGISTER);
	/* set IO pins for SDA and SCL */
	qm_pmux_select(QM_PIN_ID_6, QM_PMUX_FN_2);
	qm_pmux_select(QM_PIN_ID_7, QM_PMUX_FN_2);

	/* Configure I2C */
	I2Ccfg.address_mode = QM_I2C_7_BIT;
	I2Ccfg.mode = QM_I2C_MASTER;
	I2Ccfg.speed = QM_I2C_SPEED_STD;

	/* set the configuration through the structure and return if failure */
	if (qm_i2c_set_config(QM_I2C_0, &I2Ccfg)) {
		QM_PUTS("Error: I2C_0 config\n");
		return (errno);
	} else {
		return (0);
	}


}
/*___________________________________________________________________________*/
int readBH1750() {
	/* read the sensor data
	 * will return 0 upon success or errno
	 *
	 * set up the i2c structure for a read
	 */
	irq_read_xfer.tx=NULL;
	irq_read_xfer.tx_len=0;
	irq_read_xfer.callback=i2c_cb;
	irq_read_xfer.callback_data=&id_read;
	irq_read_xfer.rx=BHlxdata;
	irq_read_xfer.rx_len=BYTES2READ;
	irq_read_xfer.stop=true;
	i2cirq_complete=false;
	if(qm_i2c_master_irq_transfer(QM_I2C_0,&irq_read_xfer,BH1750addr))
	{
		return(errno);
	}
	/* wait for I2C to finish if it is not done */
	while(! i2cirq_complete);
	return (0);
}
/*____________________________________________________________________________*/
int initBH1750() {
	/* configure the sensor mode */

	uint8_t BHmodedata[1] = {0x10}; /* BH1750 initialization code for 1 lux resolution */
	/* will return 0 upon success or errno */

	/* set up the i2c structure for write */
	irq_write_xfer.tx=BHmodedata;
	irq_write_xfer.tx_len=sizeof(BHmodedata);
	irq_write_xfer.callback=i2c_cb;
	irq_write_xfer.callback_data=&id_write;
	irq_write_xfer.rx=NULL;
	irq_write_xfer.rx_len=0;
	irq_write_xfer.stop=true;
	i2cirq_complete=false;
	if(qm_i2c_master_irq_transfer(QM_I2C_0,&irq_write_xfer,BH1750addr))
	{
		return(errno);
	}
	/* wait for I2C to finish if it is not done */
	while(! i2cirq_complete);
	clk_sys_udelay(180000); /* 180 ms delay as per BH1750 data sheet */
	return (0);
}
/*____________________________________________________________________________*/
int main(void) {

	int rescode;

	QM_PUTS("BH1750 Sensor Test\n");
	/* configure I2C and quit if error */
	if ( (rescode=wireBegin() ) ) {
		QM_PRINTF("Error: Could not configure I2C (errno=%d)\n",rescode);
		return (1);
	}
	/* write sensor mode and quit if error */
	if ( (rescode=initBH1750())) {
		QM_PRINTF("Error: Could not write BH1750 mode (errno=%d)\n",rescode);
		return (1);
	}
	/* read sensor and display result loop*/
	while (1) {
		if((rescode=readBH1750())) {
			QM_PRINTF("Error: Could not read BH1750 (errno=%d)\n",rescode);
			return (1);
		}
		/* translate the data */
		lux = ((BHlxdata[0] << 8) | BHlxdata[1]) / 1.2;
		QM_PRINTF("%d lux\n",lux);
		/* wait 120 ms  before the next read as per BH1750 data sheet */
		clk_sys_udelay(120000);
		/* delay 1.5 sec for the loop print - change as desired */
		clk_sys_udelay(1500000);
	}
	/* never gets here */
	return (0);
}
/*____________________________________________________________________________*/
/* I2C Callback function */
static void i2c_cb(void *data, int rc, qm_i2c_status_t status, uint32_t len)
{
	/* just set a finished flag */
	i2cirq_complete=true;
}

